/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;
import net.northking.cctp.testbug.api.service.BugRecordApiService;
import net.northking.cctp.testbug.dto.BugRecordDetailDto;
import net.northking.cctp.testbug.dto.BugRecordQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

@Api(tags = "缺陷变更记录表")
@RequestMapping("/v1")
@RestController
public class BugRecordQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(BugRecordQueryCtrl.class);

  @Autowired
  private BugRecordApiService apiService;

  /**
   * 分页查询 缺陷变更记录表 列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：缺陷变更记录表")
  @PostMapping(value = "/paging/query/BugRecords",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<BugRecordDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<BugRecordQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<BugRecordDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<BugRecordDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

  @ApiOperation(value = "导出Excel：缺陷变更记录表")
  @PostMapping(value = "/excel/export/BugRecords",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void exportExcel(
      @RequestBody @Validated QueryByPage<BugRecordQueryDto> queryByPage,
      HttpServletResponse response)
  {
    // 定义导出的Excel在客户端浏览器上提示下载的文件名，不写后缀
    String fileName = "CCTP-缺陷变更记录表";
    apiService.exportExcel(response, queryByPage, fileName);
  }

  @ApiOperation(value = "下载Excel模板：缺陷变更记录表")
  @PostMapping(value = "/excel/template/BugRecords",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void downloadTemplate(HttpServletResponse response)
  {
    String fileName = "CCTP模板-缺陷变更记录表";
    apiService.downloadTemplate(response, fileName);
  }

  @ApiOperation(value = "导入Excel：缺陷变更记录表")
  @PostMapping(value = "/excel/import/BugRecords",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  public ResultWrapper<Integer> importExcel(
      @RequestParam("file") MultipartFile file)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    int count = apiService.importExcel(file);

    return wrapper.success(count);
  }


}


