/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.dto.BugOverallTendencyChartAddDto;
import net.northking.cctp.testbug.dto.BugOverallTendencyChartDetailDto;
import net.northking.cctp.testbug.dto.BugOverallTendencyChartQueryDto;
import net.northking.cctp.testbug.dto.BugOverallTendencyChartUpdateDto;

import java.util.List;

/**
 * 跨项目缺陷趋势图Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugOverallTendencyChartApiService extends ExcelService
{
  /**
   * 查询跨项目缺陷趋势图列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugOverallTendencyChartDetailDto> pagingQuery(QueryByPage<BugOverallTendencyChartQueryDto> queryByPage);

  /**
   * 新增跨项目缺陷趋势图
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugOverallTendencyChartAddDto dto);

  /**
   * 更新跨项目缺陷趋势图
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugOverallTendencyChartUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取跨项目缺陷趋势图
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugOverallTendencyChartDetailDto findByPK(String uuid);

  /**
   * 删除跨项目缺陷趋势图
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugOverallTendencyChartAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugOverallTendencyChartUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    int deleteAll();

    List<BugOverallTendencyChartDetailDto> selectGroupByTime();
}
