/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.db.entity.BugRecord;
import net.northking.cctp.testbug.dto.BugRecordAddDto;
import net.northking.cctp.testbug.dto.BugRecordDetailDto;
import net.northking.cctp.testbug.dto.BugRecordQueryDto;
import net.northking.cctp.testbug.dto.BugRecordUpdateDto;

import java.util.List;

/**
 * 缺陷变更记录表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugRecordApiService extends ExcelService
{
  /**
   * 查询缺陷变更记录表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugRecordDetailDto> pagingQuery(QueryByPage<BugRecordQueryDto> queryByPage);

  /**
   * 新增缺陷变更记录表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugRecordAddDto dto);

  /**
   * 更新缺陷变更记录表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugRecordUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取缺陷变更记录表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugRecordDetailDto findByPK(String uuid);

  /**
   * 删除缺陷变更记录表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugRecordAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugRecordUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  /**
   * 根据bugId查询变更记录信息
   * @param bugId
   * @return
   */
    List<BugRecordDetailDto> queryByBugId(String bugId);

  /**
   * 根据租户id查询变更记录信息
   * @param tenantId
   * @return
   */
    List<BugRecordDetailDto> queryByTenantId(String tenantId);

  /**
   * 查询缺陷变更记录
   * @param bugIds
   * @return
   */
    List<BugRecordDetailDto> queryByBugIds(List<String> bugIds);

  void pubAdd(BugRecord bugRecordAddDto);
}
