/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.platform.dto.comlibrary;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import net.northking.cctp.platform.db.entity.CpComponentParamter;

import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
* Restful接口--资源详情--数据传输对象：组件清单
*
* createdate:  2022-08-29 10:13:11  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@JsonSerialize
@ApiModel(description = "组件清单")
public class CpComponentListDetailDto implements Serializable
{
    /**
     * 主键<br>
     */
    @ApiModelProperty("主键")
    @Size(max = 32, message="主键-id: 数据长度不能 > 32" )
    private String id;
    /**
     * 所属组件库<br>
     */
    @ApiModelProperty("所属组件库")
    @Size(max = 32, message="所属组件库-libraryId: 数据长度不能 > 32" )
    private String libraryId;
    /**
     * 组件名<br>
     */
    @ApiModelProperty("组件名")
    @Size(max = 100, message="组件名-comName: 数据长度不能 > 100" )
    private String comName;
    /**
     * 组件别名<br>
     */
    @ApiModelProperty("组件别名")
    @Size(max = 100, message="组件别名-comAlias: 数据长度不能 > 100" )
    private String comAlias;

    /**
     * 组件属性<br>
     */
    @ApiModelProperty("组件属性:0-公共,-接口,2-pcBS,3-pcCS,4-安卓,5-ios")
    @Size(max = 100, message="组件别名-comAlias: 数据长度不能 > 10" )
    private String comAttributes;
    /**
     * 使用类别：0-基础，1-插件2-函数<br>
     */
    @ApiModelProperty("使用类别：0-基础，1-插件2-函数")
    @Size(max = 5, message="使用类别：0-基础，1-插件2-函数-comCategory: 数据长度不能 > 5" )
    private String comCategory;
    /**
     * 组建介绍<br>
     */
    @ApiModelProperty("组建介绍")
    @Size(max = 2000, message="组建介绍-comIntro: 数据长度不能 > 2000" )
    private String comIntro;
    /**
     * 是否推荐使用0-推荐，1-不推荐<br>
     */
    @ApiModelProperty("是否推荐使用0-推荐，1-不推荐")
    private Integer isDeprecated;
    
    /**
     * 创建人<br>
     */
    @ApiModelProperty("创建人")
    @Size(max = 32, message="创建人-createdBy: 数据长度不能 > 32" )
    private String createdBy;
    /**
     * 创建时间<br>
     */
    @ApiModelProperty("创建时间")
    private Date createdTime;
    /**
     * 修改人<br>
     */
    @ApiModelProperty("修改人")
    @Size(max = 32, message="修改人-updatedBy: 数据长度不能 > 32" )
    private String updatedBy;
    /**
     * 修改时间<br>
     */
    @ApiModelProperty("修改时间")
    private Date updatedTime;
    /**
     * 租户id<br>
     */
    @ApiModelProperty("租户id")
    @Size(max = 32, message="租户id-tenantId: 数据长度不能 > 32" )
    private String tenantId;

    @ApiModelProperty("参数list")
    private List<CpComponentParamter> paramList;

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getLibraryId() {
        return libraryId;
    }

    public void setLibraryId(String libraryId) {
        this.libraryId = libraryId;
    }

    public String getComName() {
        return comName;
    }

    public void setComName(String comName) {
        this.comName = comName;
    }

    public String getComAlias() {
        return comAlias;
    }

    public void setComAlias(String comAlias) {
        this.comAlias = comAlias;
    }

    public String getComCategory() {
        return comCategory;
    }

    public void setComCategory(String comCategory) {
        this.comCategory = comCategory;
    }

    public String getComIntro() {
        return comIntro;
    }

    public void setComIntro(String comIntro) {
        this.comIntro = comIntro;
    }

    public Integer getIsDeprecated() {
        return isDeprecated;
    }

    public void setIsDeprecated(Integer isDeprecated) {
        this.isDeprecated = isDeprecated;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public Date getCreatedTime() {
        return createdTime;
    }

    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    public String getUpdatedBy() {
        return updatedBy;
    }

    public void setUpdatedBy(String updatedBy) {
        this.updatedBy = updatedBy;
    }

    public Date getUpdatedTime() {
        return updatedTime;
    }

    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    public String getTenantId() {
        return tenantId;
    }

    public void setTenantId(String tenantId) {
        this.tenantId = tenantId;
    }

    public String getComAttributes() {
        return comAttributes;
    }

    public void setComAttributes(String comAttributes) {
        this.comAttributes = comAttributes;
    }

    public List<CpComponentParamter> getParamList() {
        return paramList;
    }

    public void setParamList(List<CpComponentParamter> paramList) {
        this.paramList = paramList;
    }
}